#include <me218_c32.h>
#include <stdio.h>
#include <timers12.h>
#include <ses.h>
#include "PWMS12.h"
#include "BITDEFS.h"
#include "ADS12.h"

/*---------------- Module Defines ---------------------------*/
//The wheel speeds when the robot is cruising around the perimeter
#define HIGHCRUISINGSPEED 10
#define LOWCRUISINGSPEED 9 
#define ONESEC 250
#define LIGHT_HYSTERESIS_CONSTANT 25
#define BEACON_READ_PERIOD ONESEC/1
#define DOORLOCK_WAIT_TIME ONESEC/1
#define INITIAL_WAIT_TIME 3*ONESEC
#define INITIAL_BEACON_WAIT_TIME ONESEC/2
#define FOUND_BEACON_COUNT_NUMBER 10
#define LIGHT_SENSOR_THRESHOLD 530
/*---------------- Function Prototypes ----------------------*/
unsigned char TestForTmr5(EVENT_PARAM);
void RespToTmr5(SERVICE_PARAM);

unsigned char TestForBeacon(EVENT_PARAM);
void RespToBeacon(SERVICE_PARAM);
unsigned char TestForKey(EVENT_PARAM);
void RespToKey(SERVICE_PARAM);
void RespToBeacon(SERVICE_PARAM);
void EnterOrientationState();
void EnterWallTravelingState();
void EnterWaitingState();
void SetMotor1Speed(int speed);
void SetMotor2Speed(int speed);
void SetMotor1CW();
void SetMotor1CCW();
void SetMotor2CW();
void SetMotor2CCW();

/*---------------- Module Code -----------------------------*/

void main (void)
{
  /* initialize the SES module to operate in round robin mode, with no
  timer delay */
  SES_Init(SES_ROUND_ROBIN);
  TMRS12_Init(TMRS12_RATE_4MS);      //initialize timer module

  /* register each pair of Events and Services routines*/
  SES_Register(TestForTmr5, RespToTmr5);  
  SES_Register(TestForBeacon, RespToBeacon);

  
  /*Initialize ports 0-2 as PWM ports*/
  PWMS12_Init();
  
  /*Initialize T ports 5 and 6 to be digital outputs for the h-bridge*/
  DDRT |= BIT5HI;//Initialize PortT5 to be an output           
  PTT |= BIT5HI;//set the motor1 to turn clockwise 
  DDRT |= BIT6HI;//Initialize PortT6 to be an output           
  PTT |= BIT6HI;//set the motor2 to turn clockwise 
  
  /*Initialize M port 0 and 1to be an output*/
  DDRM |= BIT0HI;//Initialize PortM0 to be an output           
  PTM |= BIT0HI;//set the motor to be retracting
  DDRM |= BIT1HI;//Initialize PortM1 to be an output           
  PTM |= BIT1HI;//enable the motor
  
  /*Initialize M port 4 to control resetting the beacon sensing circuit*/
  DDRM |= BIT4HI;//Initialize PortM0 to be an output           
  //reset the beacon sensing circuit
  PTM |= BIT4HI;
  PTM &= BIT4LO;
  
  /*Initialize AD port 0 to be an analog input*/
  if(ADS12_Init("OOOOOOOA")==ADS12_OK){
    printf("successfully initiated ADS12 \r\n");
  }else {
    printf("ADS12 failed\r\n");
    return;
  }  
    
  //begin orientation routine
  //EnterOrientationState();
  //begin waiting state
  //EnterWaitingState(); 
  //EnterWallTravelingState();
 
  
  TMRS12_InitTimer(5,INITIAL_WAIT_TIME);  
  //Initialize timer 2 to control beacon sensing period
  TMRS12_InitTimer(2, BEACON_READ_PERIOD);
  //Initialize timer 3 to set a wait time before starting beacon sensing
  TMRS12_InitTimer(3, INITIAL_BEACON_WAIT_TIME+INITIAL_WAIT_TIME);  
  //Initialize timer 7 to set 2 minute max time 
  TMRS12_InitTimer(7, ONESEC*120); 
                      
  /* enter an infinite loop and handle the events */
	while (1) {	
		SES_HandleEvents();
	}
}

//********************************************************************
//State functions
//********************************************************************/
void EnterWaitingState(){
  //disable wheel motors 1 and 2
  SetMotor1Speed(0);
  SetMotor2Speed(0);
}
void EnterWallTravelingState(){
  SetMotor1CW();
  SetMotor2CW();
  SetMotor1Speed(HIGHCRUISINGSPEED);
  SetMotor2Speed(LOWCRUISINGSPEED);
}

void EnterOrientationState(){
  SetMotor1CW();
  SetMotor2CCW();
  SetMotor1Speed(5);
  SetMotor2Speed(5);    
}


//********************************************************************
//Motor functions
//********************************************************************/

void SetMotor1Speed(int speed){
  if(speed>10){
    printf("Error - motor requested motor speed is greater than 10");
    speed = 10;
  }
  PWMS12_SetDuty(10*speed, 0);    
}
void SetMotor2Speed(int speed){
  if(speed>10){
    printf("Error - motor requested motor speed is greater than 10");
    speed = 10;
  }
  PWMS12_SetDuty(10*speed, 1);    
}
void SetMotor1CW(){
  PTT |= BIT5HI;//set the motor1 to turn clockwise   
}

void SetMotor1CCW(){
  PTT &= BIT5LO;//set the motor1 to turn counter clockwise   
}
void SetMotor2CW(){
  PTT |= BIT6HI;//set the motor1 to turn clockwise   
}

void SetMotor2CCW(){
  PTT &= BIT6LO;//set the motor1 to turn counter clockwise   
}

void SetDoor1CW(){
  PTM |= BIT0HI;//set the motor1 to turn clockwise   
}

void SetDoor1CCW(){
  PTM &= BIT0LO;//set the motor1 to turn counter clockwise   
}
void SetDoor2CW(){
  PTM |= BIT1HI;//set the motor1 to turn clockwise   
}

void SetDoor2CCW(){
  PTM &= BIT1LO;//set the motor1 to turn counter clockwise   
}



//********************************************************************
//Event and Service Functions 
//********************************************************************/

unsigned char TestForBeacon(EVENT_PARAM)
{  	
  static int printCount = 0;
  int lightLevel;
  //printf("(1) printCount entering TestForBeacon = %d \r\n",printCount);  
  if(TMRS12_IsTimerExpired(3)) {
    //printf("entered first for loop\r\n");
    if(TMRS12_IsTimerExpired(2)) {
      //printf("entered timer2 loop\r\n");
      //restart the timer
      TMRS12_InitTimer(2, ONESEC/100);                    
      
      //read the light level
      lightLevel = ADS12_ReadADPin(0);
      if(printCount > 50){
        printf("Port AD0 = %d \n\r",lightLevel);
        printCount = 0;
      }else{
        //printf("did not enter print loop. printCount = %d\r\n",printCount);
        printCount++;
        //printf("new printloop = %d \r\n",printCount);
      }
      
      //clear the capacitor from the beacon sensor resetting the peak value
      PTM |= BIT4HI;
      PTM &= BIT4LO;
      
      //update maxLightLevel or check if light level has started to drop  
      if(lightLevel>LIGHT_SENSOR_THRESHOLD){
        printf("BEACON FOUND lightLevel = %d\r\n",lightLevel); 
        return 1;
      }
    }else{
      //printf("[Tmr3,Tmr5] = [%d,%d]\r\n",TMRS12_IsTimerExpired(3),TMRS12_IsTimerExpired(5));
    }
  }
  return 0;                 
}

void RespToBeacon(SERVICE_PARAM)
{                       
  printf("EnterWaitingState\r\n");
  EnterWaitingState();
  //EnterWallTravelingState();  
}

unsigned char TestForTmr5(EVENT_PARAM)
{
  if(TMRS12_IsTimerExpired(5)) {
   
   return 1;
  }
  return 0;
}

void RespToTmr5(SERVICE_PARAM)
{                         
  EnterOrientationState();
  printf("Timer 5 is expired again \r\n");
  TMRS12_ClearTimerExpired(5);
}

